// src/index.ts
import Select from "react-select";

// src/withSelectFetch.tsx
import { useComponents } from "react-select-async-paginate";

// src/useSelectFetch.ts
import { useAsyncPaginate } from "react-select-async-paginate";

// src/useMapToAsyncPaginate.ts
import { useCallback, useMemo } from "react";
import { checkIsResponse } from "react-select-async-paginate";

// src/stringifyParams.ts
import qs from "qs";
var stringifyParams = (params) => qs.stringify(params, {
  arrayFormat: "repeat"
});

// src/get.ts
var get = async (url, params) => {
  const paramsStr = stringifyParams(params);
  const response = await fetch(`${url}?${paramsStr}`, {
    credentials: "same-origin"
  });
  if (response.status >= 400) {
    throw new Error("Failed to fetch");
  }
  const responseJSON = await response.json();
  return responseJSON;
};

// src/useMapToAsyncPaginate.ts
var errorText = '[react-select-fetch] response should be an object with "options" prop, which contains array of options. Also you can use `mapResponse` param';
var defaultResponseMapper = (response) => {
  if (checkIsResponse(response)) {
    return response;
  }
  throw new Error(errorText);
};
var useMapToAsyncPaginate = (selectFetchParams) => {
  const {
    url,
    queryParams = {},
    searchParamName = "search",
    pageParamName = "page",
    offsetParamName = "offset",
    mapResponse = defaultResponseMapper,
    get: get2 = get,
    initialPage = 1,
    defaultInitialPage = 2
  } = selectFetchParams;
  const additional = useMemo(
    () => ({
      page: initialPage
    }),
    [initialPage]
  );
  const defaultAdditional = useMemo(
    () => ({
      page: defaultInitialPage
    }),
    [defaultInitialPage]
  );
  const loadOptions = useCallback(
    async (search, prevOptions, currentAdditional) => {
      if (currentAdditional === void 0) {
        throw new Error();
      }
      const { page } = currentAdditional;
      const params = {
        ...queryParams
      };
      if (searchParamName) {
        params[searchParamName] = search;
      }
      if (pageParamName) {
        params[pageParamName] = page;
      }
      if (offsetParamName) {
        params[offsetParamName] = prevOptions.length;
      }
      const result = await get2(url, params);
      const response = mapResponse(result, {
        search,
        prevPage: page,
        prevOptions
      });
      return {
        ...response,
        additional: {
          page: page + 1
        }
      };
    },
    [
      url,
      queryParams,
      searchParamName,
      pageParamName,
      offsetParamName,
      mapResponse,
      get2
    ]
  );
  return {
    loadOptions,
    additional,
    defaultAdditional
  };
};

// src/useSelectFetch.ts
var useSelectFetch = (params, deps = []) => {
  const mappedParams = useMapToAsyncPaginate(params);
  const result = useAsyncPaginate(
    {
      ...params,
      ...mappedParams
    },
    deps
  );
  return result;
};

// src/withSelectFetch.tsx
import { jsx } from "react/jsx-runtime";
var defaultCacheUniqs = [];
var defaultComponents = {};
function withSelectFetch(SelectComponent) {
  function WithSelectFetch(props) {
    const {
      components = defaultComponents,
      selectRef = void 0,
      cacheUniqs = defaultCacheUniqs,
      ...rest
    } = props;
    const asyncPaginateProps = useSelectFetch(rest, cacheUniqs);
    const processedComponents = useComponents(
      components
    );
    return /* @__PURE__ */ jsx(
      SelectComponent,
      {
        ...props,
        ...asyncPaginateProps,
        components: processedComponents,
        ref: selectRef
      }
    );
  }
  return WithSelectFetch;
}

// src/useSelectFetchBase.ts
import { useAsyncPaginateBase } from "react-select-async-paginate";
var useSelectFetchBase = (params, deps = []) => {
  const mappedParams = useMapToAsyncPaginate(params);
  const result = useAsyncPaginateBase(
    {
      ...params,
      ...mappedParams
    },
    deps
  );
  return result;
};

// src/index.ts
var SelectFetch = withSelectFetch(Select);
export {
  SelectFetch,
  useMapToAsyncPaginate,
  useSelectFetch,
  useSelectFetchBase,
  withSelectFetch
};
//# sourceMappingURL=index.js.map