import { ReactElement, Ref } from 'react';
import { GroupBase, OptionsOrGroups, Props, SelectInstance } from 'react-select';
import { Response, UseAsyncPaginateParams, UseAsyncPaginateBaseParams, ComponentProps, UseAsyncPaginateBaseResult, UseAsyncPaginateResult } from 'react-select-async-paginate';

type Additional = {
    page: number;
};
type MapResponsePayload<OptionType, Group extends GroupBase<OptionType>> = {
    search: string;
    prevPage: number;
    prevOptions: OptionsOrGroups<OptionType, Group>;
};
type MapResponse<OptionType, Group extends GroupBase<OptionType>> = (responseRaw: unknown, payload: MapResponsePayload<OptionType, Group>) => Response<OptionType, Group, Additional>;
type Get = <Response>(url: string, params: {
    [key: string]: unknown;
}) => Promise<Response>;
type UseSelectFetchMapParams<OptionType, Group extends GroupBase<OptionType>> = {
    url: string;
    queryParams?: {
        [key: string]: unknown;
    };
    searchParamName?: string | null;
    pageParamName?: string | null;
    offsetParamName?: string | null;
    mapResponse?: MapResponse<OptionType, Group>;
    get?: Get;
    initialPage?: number;
    defaultInitialPage?: number;
};
type UseSelectFetchParams<OptionType, Group extends GroupBase<OptionType>> = UseSelectFetchMapParams<OptionType, Group> & Partial<UseAsyncPaginateParams<OptionType, Group, Additional>>;
type UseSelectFetchBaseParams<OptionType, Group extends GroupBase<OptionType>> = UseSelectFetchParams<OptionType, Group> & Partial<UseAsyncPaginateBaseParams<OptionType, Group, Additional>> & {
    inputValue: string;
    menuIsOpen: boolean;
};
type SelectFetchProps<OptionType, Group extends GroupBase<OptionType>, IsMulti extends boolean> = Props<OptionType, IsMulti, Group> & UseSelectFetchParams<OptionType, Group> & ComponentProps<OptionType, Group, IsMulti>;
type SelectFetchType = <OptionType, Group extends GroupBase<OptionType>, IsMulti extends boolean = false>(props: SelectFetchProps<OptionType, Group, IsMulti>) => ReactElement;

type SelectComponentType = <Option = unknown, IsMulti extends boolean = boolean, Group extends GroupBase<Option> = GroupBase<Option>>(props: Props<Option, IsMulti, Group> & {
    ref?: Ref<SelectInstance<Option, IsMulti, Group>>;
}) => ReactElement;
declare function withSelectFetch(SelectComponent: SelectComponentType): SelectFetchType;

declare const useSelectFetchBase: <OptionType, Group extends GroupBase<OptionType>>(params: UseSelectFetchBaseParams<OptionType, Group>, deps?: ReadonlyArray<unknown>) => UseAsyncPaginateBaseResult<OptionType, Group>;

declare const useSelectFetch: <OptionType, Group extends GroupBase<OptionType>>(params: UseSelectFetchParams<OptionType, Group>, deps?: ReadonlyArray<unknown>) => UseAsyncPaginateResult<OptionType, Group>;

declare const useMapToAsyncPaginate: <OptionType, Group extends GroupBase<OptionType>>(selectFetchParams: UseSelectFetchMapParams<OptionType, Group>) => UseAsyncPaginateParams<OptionType, Group, Additional>;

declare const SelectFetch: SelectFetchType;

export { type Additional, type Get, type MapResponse, type MapResponsePayload, SelectFetch, type SelectFetchProps, type SelectFetchType, type UseSelectFetchBaseParams, type UseSelectFetchMapParams, type UseSelectFetchParams, useMapToAsyncPaginate, useSelectFetch, useSelectFetchBase, withSelectFetch };
